# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-04-09 20:27
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='CommandOutput',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('output', models.TextField(null=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
        ),
        migrations.CreateModel(
            name='ManagementCommand',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(db_index=True, help_text=b'The full path of the command', max_length=128)),
                ('status', models.CharField(help_text=b'The current status of the command', max_length=32)),
                ('last_run', models.DateTimeField(help_text=b'The last time this command was run', null=True)),
                ('last_success', models.DateTimeField(help_text=b'The last time this command was run successfully', null=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
        ),
        migrations.AddField(
            model_name='commandoutput',
            name='command',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='djenga.ManagementCommand'),
        ),
    ]
